SystemOrganization addCategory: #'MonticelloFileDirectory-Base'!
SystemOrganization addCategory: #'MonticelloFileDirectory-Test'!

MCVersionReader subclass: #McFdReader
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileDirectory-Base'!

!McFdReader methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 14:37'!
associate: tokens
	| result |
	result := Dictionary new.
	tokens pairsDo: [:key :value | 
					value isString ifFalse: [value := value collect: [:ea | self associate: ea]].
					result at: key put: value].
	^ result! !

!McFdReader methodsFor: 'as yet unclassified' stamp: 'MU 12/5/2007 13:45'!
createDependencies! !

!McFdReader methodsFor: 'as yet unclassified' stamp: 'MU 12/5/2007 13:45'!
createPackage! !

!McFdReader methodsFor: 'as yet unclassified' stamp: 'MU 12/5/2007 13:46'!
createVersion! !

!McFdReader methodsFor: 'as yet unclassified' stamp: 'pe 12/11/2007 23:02'!
createVersionInfo
	| aStream |
	aStream := FileStream readOnlyFileNamed: 'VersionInfo.txt'.
	info := self extractInfoFrom: (self associate: (MCScanner scanTokens: aStream contents) first).! !

!McFdReader methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 14:41'!
extractInfoFrom: dict
	^ MCVersionInfo
			name: (dict at: #name ifAbsent: [''])
			id: (UUID fromString: (dict at: #id))
			message: (dict at: #message ifAbsent: [''])
			date: ([Date fromString: (dict at: #date) ] on: Error do: [ :ex | ex return: nil ])
			time: ([ Time fromString:(dict at: #time)] on: Error do: [ :ex | ex return: nil ])
			author: (dict at: #author ifAbsent: [''])
			ancestors: ((dict at: #ancestors) collect: [:ea | self extractInfoFrom: ea])
			stepChildren: ((dict at: #stepChildren ifAbsent: [#()]) collect: [:ea | self extractInfoFrom: ea]).! !

MCWriter subclass: #McFdCreator
	instanceVariableNames: 'packageDirectory versionDirectory extensionDirectory classCategories version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileDirectory-Base'!

!McFdCreator class methodsFor: 'as yet unclassified' stamp: 'pe 12/12/2007 11:54'!
newForVersion: aVersion at: aDirectory 
	^ (self new)
		version: aVersion;
		createPackageDirectoryIn: aDirectory;
		yourself! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/11/2007 12:02'!
classCategories
	^ classCategories! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/11/2007 12:12'!
classCategories: anObject
	classCategories := anObject! !

!McFdCreator methodsFor: 'creation' stamp: 'pe 12/12/2007 12:25'!
createAllForVersion: aVersion at: aDirectory 
	| ordCol |
	ordCol := OrderedCollection new.
	aVersion package packageInfo systemCategories do: 
			[:each | 
			self createDirectory: (self extractCategoryStringFrom: each asString)
				at: versionDirectory.
			ordCol add: each].
	self classCategories: ordCol.
	self createSourceCodeFiles.
	self createExtensionsFrom: aVersion at: extensionDirectory! !

!McFdCreator methodsFor: 'creation' stamp: 'pe 12/12/2007 11:44'!
createDirectory: aDirectoryName at: aDirectory 
	| directory dirString |
	dirString := aDirectory fullName , FileDirectory slash , aDirectoryName.
	directory := FileDirectory on: dirString.
	directory assureExistence.
	^directory.! !

!McFdCreator methodsFor: 'creation' stamp: 'pe 12/11/2007 15:36'!
createExtensionsFrom: aVersion at: aDirectory
	| astream classCollection|
	classCollection := OrderedCollection new.
	aVersion snapshot definitions do: 
		[:each | 
		each isClassDefinition 
			ifTrue: 
				[classCollection add: each className.
				]].
						
	aVersion snapshot definitions do:
		[:ea |
		(ea isMethodDefinition and: [(classCollection includes: (ea className))not])
			ifTrue:
				[
				astream := aDirectory forceNewFileNamed: ea className, '.st'.	
				
				aVersion snapshot definitions do:
				[:eachMethodDefinition |
				(eachMethodDefinition isMethodDefinition and: [ea className = eachMethodDefinition className])
					ifTrue:
						[
							self writeMethodDefinition: eachMethodDefinition to: astream.
						]].
				astream close.
				]]! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/12/2007 11:56'!
createPackageDirectoryIn: aDirectory 
	packageDirectory:= (self createDirectory: version package name at: aDirectory).! !

!McFdCreator methodsFor: 'creation' stamp: 'pe 12/12/2007 12:25'!
createSourceCodeFiles
	| directory |
	self writeVersionInfo.
	self writeDependenciesFrom: version at: versionDirectory.
	version snapshot definitions select: [:each | each isClassDefinition]
		thenDo: 
			[:each | 
			directory := self 
						createDirectory: (self extractCategoryStringFrom: each category)
						at: versionDirectory.
			stream := directory forceNewFileNamed: each className , '.st'.
			self writeClassDefinition: each to: stream.
			version snapshot definitions select: 
					[:eachMethod | 
					eachMethod isMethodDefinition and: [each className = eachMethod className]]
				thenDo: [:eachMethod | self writeMethodDefinition: eachMethod to: stream].
			stream close]! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/12/2007 12:06'!
extensionDirectory
	extensionDirectory ifNil:[extensionDirectory := (self createDirectory: '~Extensions' at: versionDirectory)].
	^ extensionDirectory! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/11/2007 11:56'!
extensionDirectory: anObject
	extensionDirectory := anObject! !

!McFdCreator methodsFor: 'creation' stamp: 'MU 11/21/2007 17:15'!
extractCategoryStringFrom: aString 
	^ (aString subStrings: '-') second.
! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/11/2007 11:55'!
packageDirectory
	^ packageDirectory! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/12/2007 11:36'!
version
	^ version! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/12/2007 11:36'!
version: anObject
	version := anObject! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/12/2007 12:03'!
versionDirectory
	versionDirectory ifNil: [versionDirectory := (self createDirectory: version info name at: packageDirectory).].
	^ versionDirectory! !

!McFdCreator methodsFor: 'accessing' stamp: 'pe 12/11/2007 11:56'!
versionDirectory: anObject
	versionDirectory := anObject! !

!McFdCreator methodsFor: 'write' stamp: 'MU 12/4/2007 13:03'!
writeClassDefinition: aClassDefinition to: aStream
	"stream nextPutAll: 'Latest update:', aVersion info date, aVersion info time."
	"stream cr."
	aStream
		nextPutAll: aClassDefinition source;
		nextPutAll: '!!';
		cr;
		cr.! !

!McFdCreator methodsFor: 'write' stamp: 'pe 11/28/2007 11:05'!
writeDependenciesFrom: aVersion at: aDirectory 
	| streamForDependencies |
	streamForDependencies := aDirectory forceNewFileNamed: 'Dependencies.txt'.
	streamForDependencies nextPutAll: aVersion dependencies.
	streamForDependencies close! !

!McFdCreator methodsFor: 'write' stamp: 'MU 12/4/2007 13:08'!
writeMethodDefinition: aMethodDefinition to: aStream 
	aStream
		nextPutAll: '!!';
		nextPutAll: (self writeMethodHeader: aMethodDefinition);
		cr;
		nextPutAll: aMethodDefinition source;
		nextPutAll: '!! !!';
		cr;
		cr! !

!McFdCreator methodsFor: 'write' stamp: 'MU 12/4/2007 13:08'!
writeMethodHeader: aMethodDefinition 
	^ aMethodDefinition className , ' methodsFor:' 
		, aMethodDefinition category asString , ' stamp:' 
		, aMethodDefinition timeStamp asString , '!!'! !

!McFdCreator methodsFor: 'write' stamp: 'pe 12/12/2007 12:27'!
writeVersionInfo
	| streamForVersionInfo aVersionInfoWriter |
	streamForVersionInfo := self versionDirectory 
				forceNewFileNamed: 'VersionInfo.txt'.
	aVersionInfoWriter := MCVersionInfoWriter on: streamForVersionInfo.
	aVersionInfoWriter writeVersionInfo: version info.
	
	["streamForVersionInfo nextPutAll: aVersion info summary."

	] 
			ensure: [streamForVersionInfo close]! !

MCVersion subclass: #McFdVersion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileDirectory-Base'!

!McFdVersion class methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 10:47'!
fromVersion: aVersion
	^(self new)
		package: aVersion package;
		info: aVersion info;
		snapshot: aVersion snapshot;
		dependencies: aVersion dependencies.! !

!McFdVersion methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 10:47'!
dependencies: aDependencies
	dependencies := aDependencies! !

!McFdVersion methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 10:47'!
info: aInfo
	info := aInfo! !

!McFdVersion methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 10:48'!
package: aPackage
	package := aPackage! !

!McFdVersion methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 10:48'!
snapshot: aSnapshot
	snapshot := aSnapshot! !

!McFdVersion methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 10:50'!
writerClass
	^ McFdCreator! !

TestCase subclass: #McFdTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileDirectory-Test'!

McFdTestCase subclass: #McFdFilesystemTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileDirectory-Test'!

!McFdFilesystemTest methodsFor: 'as yet unclassified' stamp: 'pe 11/27/2007 12:12'!
testCreateCategoryDirectory
	| creator aDirectory packageDirectory versionDirectory |
	creator := McFdCreator new.
	aDirectory := FileDirectory default.
	packageDirectory := creator createDirectory: self packageName
				at: aDirectory.
	versionDirectory := creator createDirectory: self versionName
				at: packageDirectory.
	
	[creator createDirectory: self categoryName at: versionDirectory.
	self assert: (versionDirectory directoryExists: self categoryName)] 
			ensure: [packageDirectory recursiveDelete]! !

!McFdFilesystemTest methodsFor: 'as yet unclassified' stamp: 'pe 12/12/2007 13:30'!
testCreateClassFile
	| creator aVersion categoryDirectory stream streamContents |
	aVersion := MCVersion package: self createPackage info: self createVersionInfo.
	creator := McFdCreator newForVersion: aVersion at: FileDirectory default.
	creator createSourceCodeFiles.
	
	[self assert: (creator versionDirectory directoryExists: self categoryName).
	categoryDirectory := creator versionDirectory directoryNamed: 'Base'.
	self assert: (categoryDirectory fileExists: 'McFdFooBar.st').
	stream := categoryDirectory fileNamed: 'McFdFooBar.st'.
	streamContents := stream contents.
	stream close.
	self assert: streamContents = self expectedClassFile] 
			ensure: 
				[creator packageDirectory recursiveDelete]! !

!McFdFilesystemTest methodsFor: 'as yet unclassified' stamp: 'pe 11/27/2007 11:46'!
testCreateDirectory
	| dir creator aPackage |
	creator := McFdCreator new.
	dir := FileDirectory default.
	aPackage := MCPackage named: self packageName.
	
	[creator createDirectory: aPackage name at: dir.
	self assert: (dir directoryExists: aPackage name)] 
			ensure: [dir deleteDirectory: aPackage name]! !

!McFdFilesystemTest methodsFor: 'as yet unclassified' stamp: 'pe 12/12/2007 13:31'!
testCreateVersionDirectory
	| creator aDirectory packageDirectory |
	creator := McFdCreator new.
	aDirectory := FileDirectory default.

	packageDirectory := creator createDirectory: self packageName at: aDirectory.
	
	[creator createDirectory: self versionName at: packageDirectory.
	self assert: (packageDirectory directoryExists: self versionName)
	] ensure: [packageDirectory recursiveDelete]
! !

!McFdFilesystemTest methodsFor: 'as yet unclassified' stamp: 'MU 11/21/2007 17:07'!
testExtractCategoryString
	| creator aString resultString |
	aString := 'Spaet-Abends'.
	creator := McFdCreator new.
	resultString := creator extractCategoryStringFrom: aString.
	self assert: 'Abends' = resultString.
	! !

McFdTestCase subclass: #McFdPackageTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileDirectory-Test'!

!McFdPackageTest methodsFor: 'as yet unclassified' stamp: 'MU 11/21/2007 13:16'!
testCreateMethod
	(Smalltalk at: #McFdFooBar) methodDictionary at: #m1 ifAbsent: [self fail] 
	! !

!McFdPackageTest methodsFor: 'as yet unclassified' stamp: 'MU 11/21/2007 13:16'!
testCreatePackage
	Smalltalk at: #McFdFooBar ifAbsent: [self fail].
	
	! !

!McFdTestCase methodsFor: 'as yet unclassified' stamp: 'pe 11/27/2007 12:08'!
categoryName
	^ 'Base'! !

!McFdTestCase methodsFor: 'as yet unclassified' stamp: 'pe 11/27/2007 12:08'!
classCategoryName
	^ self packageName , '-' , self categoryName! !

!McFdTestCase methodsFor: 'creation' stamp: 'pe 11/28/2007 11:23'!
createPackage
^ MCPackage named: self packageName! !

!McFdTestCase methodsFor: 'creation' stamp: 'pe 12/12/2007 12:58'!
createVersionInfo
	^ MCVersionInfo 
		name: self versionName
		id: '123'
		message: 'message'
		date: Date today
		time: Time noon
		author: 'wir'
		ancestors: #().! !

!McFdTestCase methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 10:04'!
expectedClassFile
	^ 'Object subclass: #McFdFooBar
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''McFdFooBar-Base''!!

!!McFdFooBar methodsFor:as yet unclassified stamp:MU 12/7/2007 10:04!!
m1 ^ 0!! !!

'! !

!McFdTestCase methodsFor: 'as yet unclassified' stamp: 'MU 12/7/2007 12:45'!
expectedVersionInfo
	^  'ancestors: #()
stepChildren: #()
id: 123
name: Versionname
date: 7 December 2007
time: 12:00 pm
author: wir' ! !

!McFdTestCase methodsFor: 'as yet unclassified' stamp: 'MU 11/21/2007 15:07'!
packageName
	^ 'McFdFooBar'! !

!McFdTestCase methodsFor: 'as yet unclassified' stamp: 'MU 11/21/2007 14:15'!
setUp
	Object 
		subclass: #McFdFooBar
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: self classCategoryName.
	(Smalltalk at: #McFdFooBar) compile: 'm1 ^ 0'! !

!McFdTestCase methodsFor: 'as yet unclassified' stamp: 'MU 11/21/2007 14:20'!
tearDown
	(Smalltalk at: #McFdFooBar) removeFromSystemUnlogged.
	
	Smalltalk organization removeCategory: self classCategoryName asSymbol! !

!McFdTestCase methodsFor: 'as yet unclassified' stamp: 'pe 11/27/2007 11:30'!
versionName
	^'McFdFooBar-wir.30'! !

McFdTestCase subclass: #McFdVersionInfoTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileDirectory-Test'!

!McFdVersionInfoTest methodsFor: 'as yet unclassified' stamp: 'pe 12/12/2007 13:33'!
testWriteVersionToFile
	| aVersionInfo streamForVersionInfo aVersion creator streamContents |
	aVersionInfo := self createVersionInfo.
	aVersion := MCVersion package: self createPackage info: aVersionInfo.
	creator := McFdCreator newForVersion: aVersion at: FileDirectory default.
	
	[creator writeVersionInfo.
	self assert: (creator versionDirectory fileExists: 'VersionInfo.txt').
	streamForVersionInfo := creator versionDirectory fileNamed: 'VersionInfo.txt'.
	streamContents := streamForVersionInfo contents.
	streamForVersionInfo close.
	self assert: streamContents = self expectedVersionInfo] 
			ensure: 
				[creator packageDirectory recursiveDelete]! !
